/*

Copyright 2012 Adam Reichold

This file is part of qpdfview.

qpdfview is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

qpdfview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DOCUMENTCONTROLLER_H
#define DOCUMENTCONTROLLER_H

#include <QtCore>
#include <QtGui>

#include <poppler-qt4.h>

class DocumentController : public QObject
{
    Q_OBJECT

public:
    enum ScaleModes { ScaleFactorMode, FitToPageMode, FitToPageWidthMode };
    enum DisplayModes { PagingMode, ScrollingMode, DoublePagingMode, DoubleScrollingMode };
    enum RotationModes { DoNotRotateMode, RotateBy90Mode, RotateBy180Mode, RotateBy270Mode };

    explicit DocumentController(QObject *parent = 0);
    ~DocumentController();

    void loadSettings();
    void storeSettings();


    bool loadFile(const QString &filePath);

    bool reload();
    bool saveCopy(const QString &filePath);


    int pageCount() const;

    qreal pageWidth() const;
    qreal pageHeight() const;

    QImage renderPage(const qreal &resolutionX, const qreal &resolutionY) const;
    QImage renderPage(const int &page, const qreal &resolutionX, const qreal &resolutionY) const;


    const QString &filePath() const { return _filePath; }

    const int &page() const { return _page; }

    const ScaleModes &scaleMode() const { return _scaleMode; }
    const DisplayModes &displayMode() const { return _displayMode; }
    const RotationModes &rotationMode() const { return _rotationMode; }

    const qreal &scaleFactor() const { return _scaleFactor; }
    
signals:
    void documentChanged(QString filePath);

    void pageChanged(int page);

    void scaleModeChanged(DocumentController::ScaleModes scaleMode);
    void displayModeChanged(DocumentController::DisplayModes displayMode);
    void rotationModeChanged(DocumentController::RotationModes rotationMode);

    void scaleFactorChanged(qreal scaleFactor);
    
public slots:
    void setPage(const int &page);

    void setScaleMode(const DocumentController::ScaleModes &scaleMode);
    void setDisplayMode(const DocumentController::DisplayModes &displayMode);
    void setRotationMode(const DocumentController::RotationModes &rotationMode);

    void setScaleFactor(const qreal &scaleFactor);

    void previousPage();
    void nextPage();
    void firstPage();
    void lastPage();

private:
    QSettings settings;


    Poppler::Document *_document;

    QString _filePath;

    int _page;

    ScaleModes _scaleMode;
    DisplayModes _displayMode;
    RotationModes _rotationMode;

    qreal _scaleFactor;
    
};

#endif // DOCUMENTCONTROLLER_H
