/*

Copyright 2012 Adam Reichold

This file is part of qpdfview.

qpdfview is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

qpdfview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DOCUMENTWIDGET_H
#define DOCUMENTWIDGET_H

#include <QtCore>
#include <QtGui>

#include "documentcontroller.h"

class DocumentWidget : public QWidget {
    Q_OBJECT

public:
    explicit DocumentWidget(QWidget *parent = 0);
    ~DocumentWidget();

    DocumentController *documentController() const { return _documentController; }

public slots:
    void openDocument();

    void updateDocument(const QString &filePath);

    void updatePage(const int &page);

    void updateScaleMode(const DocumentController::ScaleModes &scaleMode);
    void updateDisplayMode(const DocumentController::DisplayModes &displayMode);
    void updateRotationMode(const DocumentController::RotationModes &rotationMode);

    void updateScaleFactor(const qreal &scaleFactor);

private slots:
    void setPageByEdit();
    void setPageByValue(const int &value);

    void setScaleModeByIndex(const int &index);
    void setScaleFactorByValue(const int &value);

    void setScalingByIndex(const int &index);

protected:
    void wheelEvent(QWheelEvent *wheelEvent);
    void resizeEvent(QResizeEvent *resizeEvent);
    void paintEvent(QPaintEvent *paintEvent);

private:
    QSettings settings;

    QPushButton *openDocumentButton;

    QLabel *pageLabel;
    QLineEdit *pageLineEdit;
    QIntValidator *pageValidator;
    QLabel *pageCountLabel;

    QPushButton *previousPageButton;
    QPushButton *nextPageButton;
    QPushButton *firstPageButton;
    QPushButton *lastPageButton;

    /*QLabel *scaleModeLabel;
    QComboBox *scaleModeComboBox;

    QLabel *scaleFactorLabel;
    QSlider *scaleFactorSlider;*/

    QLabel *scalingLabel;
    QComboBox *scalingComboBox;

    QVector<qreal> selectableScaleFactors;

    QHBoxLayout *toolBarLayout;
    QWidget *toolBar;

    QGraphicsScene *mainScene;
    QGraphicsView *mainView;

    QVBoxLayout *mainLayout;


    DocumentController *_documentController;

    // internal methods

    qreal pageWidth;
    qreal pageHeight;

    qreal fitToPageWidth;
    qreal fitToPageHeight;
    qreal fitToPage;

    void prepareScaleMode();
    void prepareDisplayMode();

    // image cache

    void renderPage(const int &page, qreal x, qreal y);

    int imageCacheSize;
    QMap<int, QGraphicsPixmapItem*> *imageCache;

    DocumentController::ScaleModes cachedScaleMode;
    DocumentController::DisplayModes cachedDisplayMode;
    DocumentController::RotationModes cachedRotationMode;

    qreal cachedScaleFactor;

};

#endif // DOCUMENTWIDGET_H
