/*

Copyright 2012 Adam Reichold

This file is part of qpdfview.

qpdfview is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

qpdfview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtCore>
#include <QtGui>

#include "documentwidget.h"
#include "printthread.h"

class MainWindow : public QMainWindow {
    Q_OBJECT
    
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void openDocument();
    void addTab();
    void previousTab();
    void nextTab();
    void closeTab();

    void reload();
    void print();
    void saveCopy();

    void fullscreen();

    void about();

    void updateDocument(const QString &filePath);

    void updateScaleMode(const DocumentController::ScaleModes &scaleMode);
    void updateDisplayMode(const DocumentController::DisplayModes &displayMode);
    void updateRotationMode(const DocumentController::RotationModes &rotationMode);

private slots:
    void updateCurrentTab(const int &index);

    void closeTabByIndex(const int &index);

    void setScaleModeByAction(QAction *scaleModeAction);
    void setDisplayModeByAction(QAction *displayModeAction);
    void setRotationModeByAction(QAction *rotationModeAction);

    void beginPrinting();
    void continuePrinting(const QString &progress);
    void endPrinting();

protected:
    void dragEnterEvent(QDragEnterEvent *);
    void dropEvent(QDropEvent *);
    void closeEvent(QCloseEvent*);

private:
    QSettings settings;

    QTabWidget *tabWidget;

    QAction *openDocumentAction;
    QAction *addTabAction;
    QAction *previousTabAction;
    QAction *nextTabAction;
    QAction *closeTabAction;

    QAction *reloadAction;
    QAction *printAction;
    QAction *saveCopyAction;

    QAction *quitAction;

    QAction *previousPageAction;
    QAction *nextPageAction;
    QAction *firstPageAction;
    QAction *lastPageAction;

    QAction *scaleFactorModeAction;
    QAction *fitToPageModeAction;
    QAction *fitToPageWidthModeAction;
    QActionGroup *scaleModeActionGroup;

    QAction *pagingModeAction;
    QAction *scrollingModeAction;
    QAction *doublePagingModeAction;
    QAction *doubleScrollingModeAction;
    QActionGroup *displayModeActionGroup;

    QAction *doNotRotateAction;
    QAction *rotateBy90Action;
    QAction *rotateBy180Action;
    QAction *rotateBy270Action;
    QActionGroup *rotationModeActionGroup;

    QAction *fullscreenAction;

    QAction *aboutAction;


    QPrinter *printer;
    PrintThread *printThread;

    QByteArray geometryBeforeFullscreen;

};

#endif // MAINWINDOW_H
