/*

Copyright 2012 Adam Reichold

This file is part of qpdfview.

qpdfview is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

qpdfview is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "printthread.h"

PrintThread::PrintThread(DocumentController *documentController, QPrinter *printer, int fromPage, int toPage, QObject *parent) : QThread(parent) {
    _documentController = documentController;

    _printer = printer;

    _fromPage = fromPage;
    _toPage = toPage;
}

void PrintThread::run() {
    _printer->setFullPage(true);
    qreal printerWidth = _printer->width();
    qreal printerHeight = _printer->height();

    emit beginPrinting();

    QPainter *painter = new QPainter();
    painter->begin(_printer);

    for(int page = _fromPage; page <= _toPage; page++) {
        emit continuePrinting(tr("Printing page %1 of %2...").arg(page).arg(_toPage-_fromPage+1));

        qreal pageWidth = _documentController->pageWidth() * _printer->physicalDpiX() / 72.0;
        qreal pageHeight = _documentController->pageHeight() * _printer->physicalDpiY() / 72.0;

        qreal fitToWidth = printerWidth / pageWidth;
        qreal fitToHeight = printerHeight / pageHeight;

        qreal fit = qMin(fitToWidth, fitToHeight);

        painter->scale(fit, fit);

        QImage image = _documentController->renderPage(page, _printer->physicalDpiX(), _printer->physicalDpiY());

        painter->drawImage(0,0,image);

        if(page != _toPage) {
            _printer->newPage();
        }
    }

    emit endPrinting();

    painter->end();
    delete painter;
}
